@@ -4,7 +4,7 @@
  * tracelib.c
  *
  * Copyright (c) 2007-2008 Eugene Pimenov (GSoC)
- * Copyright (c) 2008-2010, 2012-2015 The MacPorts Project
+ * Copyright (c) 2008-2010, 2012-2015, 2017 The MacPorts Project
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -101,7 +101,8 @@ static void peerpid_list_walk(bool (*callback)(int sock, pid_t pid, const char *
 static char *name;
 static char *sandbox;
 static size_t sandboxLength;
-static char *depends;
+static char **depends = NULL;
+static size_t dependsLength = 0;
 static int sock = -1;
 static int kq = -1;
 /* EVFILT_USER isn't available (< 10.6), use the self-pipe trick to return from
@@ -110,6 +111,8 @@ static int selfpipe[2];
 static int enable_fence = 0;
 static Tcl_Interp *interp;
 
+static mount_cs_cache_t *mount_cs_cache;
+
 /**
  * Mutex that shall be acquired to exclusively lock checking and acting upon
  * the value of kq, indicating whether the event loop has started. If it has
@@ -349,9 +352,6 @@ static int TracelibSetNameCmd(Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[
         return TCL_ERROR;
     }
 
-    // initialize the depends field, in case we don't actually have any dependencies
-    depends = NULL;
-
     return TCL_OK;
 }
 
@@ -584,6 +584,13 @@ static void sandbox_violation(int sock UNUSED, const char *path, sandbox_violati
     Tcl_UnsetVar(interp, "_sandbox_viol_path", 0);
 }
 
+/**
+ * Internal helper function to compare two strings.
+ */
+static int pointer_strcmp(const char** a, const char** b) {
+    return strcmp(*a, *b);
+}
+
 /**
  * Check whether a path is in the transitive hull of dependencies of the port
  * currently being installed and send the result of the query back to the
@@ -603,7 +610,7 @@ static void sandbox_violation(int sock UNUSED, const char *path, sandbox_violati
  */
 static void dep_check(int sock, char *path) {
     char *port = 0;
-    char *t;
+    int fs_cs = -1;
     reg_registry *reg;
     reg_entry entry;
     reg_error error;
@@ -614,10 +621,26 @@ static void dep_check(int sock, char *path) {
         answer(sock, "#");
     }
 
+#ifdef __APPLE__
+    fs_cs = fs_case_sensitive_darwin(interp, path, mount_cs_cache);
+#endif /* __APPLE__ */
+
+    if (-1 == fs_cs) {
+        fs_cs = fs_case_sensitive_fallback(interp, path, mount_cs_cache);
+    }
+
+    if (-1 == fs_cs) {
+        /*
+         * Unable to determine FS case-sensitivity.
+         * Assume the worst case (case-insensitive.)
+         */
+        fs_cs = 0;
+    }
+
     /* find the port id */
     entry.reg = reg;
     entry.proc = NULL;
-    entry.id = reg_entry_owner_id(reg, path);
+    entry.id = reg_entry_owner_id(reg, path, fs_cs);
     if (entry.id == 0) {
         /* file isn't known to MacPorts */
         answer(sock, "?");
@@ -631,17 +654,15 @@ static void dep_check(int sock, char *path) {
         answer(sock, "#");
     }
 
-    /* check our list of dependencies */
-    for (t = depends; t && *t; t += strlen(t) + 1) {
-        if (strcmp(t, port) == 0) {
-            free(port);
-            answer(sock, "+");
-            return;
-        }
+    /* check our list of dependencies; use binary search on sorted list */
+    if (NULL != bsearch(&port, depends, dependsLength, sizeof(*depends),
+                        (int (*)(const void*, const void*)) pointer_strcmp)) {
+        free(port);
+        answer(sock, "+");
+    } else {
+        free(port);
+        answer(sock, "!");
     }
-
-    free(port);
-    answer(sock, "!");
 }
 
 static int TracelibOpenSocketCmd(Tcl_Interp *in) {
@@ -710,6 +731,14 @@ static int TracelibRunCmd(Tcl_Interp *in) {
     int opensockcount = 0;
     bool break_eventloop = false;
 
+    /* (Re-)initialize mount point FS case-sensitivity cache. */
+    if (mount_cs_cache) {
+        reset_mount_cs_cache(mount_cs_cache);
+    }
+    else {
+        mount_cs_cache = new_mount_cs_cache();
+    }
+
     pthread_mutex_lock(&evloop_mutex);
     /* bring all variables into a defined state so the cleanup code can be
      * called from anywhere */
@@ -915,6 +944,14 @@ error_locked:
     // wake up any waiting threads in TracelibCloseSocketCmd
     pthread_cond_broadcast(&evloop_signal);
 
+    /* Free mount_cs_cache object. */
+    if (mount_cs_cache) {
+        reset_mount_cs_cache(mount_cs_cache);
+
+        free(mount_cs_cache);
+        mount_cs_cache = NULL;
+    }
+
     return retval;
 }
 
@@ -934,7 +971,11 @@ static int TracelibCleanCmd(Tcl_Interp *interp UNUSED) {
         safe_free(name);
     }
 
+    for (size_t i = 0; i < dependsLength; ++i) {
+        safe_free(depends[i]);
+    }
     safe_free(depends);
+    dependsLength = 0;
 
     enable_fence = 0;
     return TCL_OK;
@@ -970,27 +1011,53 @@ static int TracelibCloseSocketCmd(Tcl_Interp *interp UNUSED) {
 }
 
 static int TracelibSetDeps(Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]) {
-    char *t, * d;
-    size_t l;
+    Tcl_Obj **objects;
+    int length;
     if (objc != 3) {
         Tcl_WrongNumArgs(interp, 2, objv, "number of arguments should be exactly 3");
         return TCL_ERROR;
     }
 
-    d = Tcl_GetString(objv[2]);
-    l = strlen(d);
-    depends = malloc(l + 2);
-    if (!depends) {
+    if (TCL_OK != Tcl_ListObjGetElements(interp, objv[2], &length, &objects)) {
+        return TCL_ERROR;
+    }
+
+    /* When called twice, do not leak memory */
+    if (depends) {
+        for (size_t i = 0; i < dependsLength; ++i) {
+            free(depends[i]);
+        }
+        free(depends);
+    }
+    depends = NULL;
+    dependsLength = 0;
+
+    /* Allocate memory as needed */
+    if (NULL == (depends = malloc(length * sizeof(*depends)))) {
         Tcl_SetResult(interp, "memory allocation failed", TCL_STATIC);
         return TCL_ERROR;
     }
-    depends[l + 1] = 0;
-    strlcpy(depends, d, l + 2);
-    for (t = depends; *t; ++t)
-        if (*t == ' ') {
-            *t++ = 0;
+    /* Copy all objects over */
+    for (int i = 0; i < length; ++i) {
+        if (NULL == (depends[i] = strdup(Tcl_GetString(objects[i])))) {
+            /* Allocation failed, clean up what we have so far */
+            for (int j = 0; j < i; ++j) {
+                free(depends[j]);
+            }
+            free(depends);
+            depends = NULL;
+            dependsLength = 0;
+            Tcl_SetResult(interp, "memory allocation failed", TCL_STATIC);
+            return TCL_ERROR;
         }
 
+        dependsLength++;
+    }
+
+    /* Sort all dependencies so we can use binary searching */
+    qsort(depends, dependsLength, sizeof(*depends),
+          (int (*)(const void*, const void*)) pointer_strcmp);
+
     return TCL_OK;
 }
 
