@@ -160,12 +160,17 @@ proc portfetch::checksites {sitelists mirrorfile} {
             continue
         }
         global ${listname}.mirror_subdir
-        # add the specified global and user-defined mirrors
-        set sglobal [lindex $extras 0]; set senv [lindex $extras 1]
         set full_list [set $listname]
-        append full_list " $sglobal"
-        if {[info exists env($senv)]} {
-            set full_list [concat $env($senv) $full_list]
+        # add the specified global and user-defined mirrors
+        set sglobal ""
+        set senv ""
+        if {[llength $extras] >= 2} {
+            set sglobal [lindex $extras 0]
+            set senv [lindex $extras 1]
+            append full_list " $sglobal"
+            if {[info exists env($senv)]} {
+                set full_list [concat $env($senv) $full_list]
+            }
         }
 
         set site_list [list]
